@@ -1,3 +1,4 @@
+from io import StringIO
 import json
 import os
 import signal
@@ -21,8 +22,10 @@ FLG = flags_.FLAGS
 env = os.environ
 py_env = load.py_env
 
-kvmsg = lambda kw: '  ' + '\n  '.join(['%s: %s' % (k, str(v)) for k, v in kw.items()])
-kvprint = lambda l, msg, kw: print('[%s] %s\n%s' % (l, msg, kvmsg(kw)))
+
+def kvmsg(kw): return '  ' + '\n  '.join(['%s: %s' % (k, str(v)) for k, v in kw.items()])
+def kvprint(l, msg, kw): return print('[%s] %s\n%s' % (l, msg, kvmsg(kw)))
+
 
 notifier = [None]
 
@@ -54,6 +57,7 @@ class App:
 
     is_initted = False
     name = sys.argv[0]
+    selected_action = None
 
     def die(self, msg, **kw):
         App().warn(msg, **kw)
@@ -143,7 +147,7 @@ def set_dirs():
     #        if k.startswith(p)
     #    ]
     # )
-    ## an app (like build may allow to set --da_dir, overruling env:
+    # an app (like build may allow to set --da_dir, overruling env:
     # d['da_dir'] = getattr(FLG, 'da_dir', env.get('DA_DIR', '.'))
     # we are used e.g. at construct/prepare.sh w/o DA:
     # c = 'CONDA_PREFIX'
@@ -152,7 +156,7 @@ def set_dirs():
     # return d
 
 
-command_name = lambda: sys.argv[0].rsplit('/', 1)[-1].replace('.py', '')
+def command_name(): return sys.argv[0].rsplit('/', 1)[-1].replace('.py', '')
 
 
 def set_app(name, log):
@@ -164,7 +168,12 @@ def set_app(name, log):
     set_dirs()
     # [setattr(app, k, v) for k, v in dirs(name).items()]
     # allows raise app.die(msg, **kw):
-    app.die = app_die(app)
+
+    def die(msg, **kw):
+        """Application decided to bail out"""
+        raise DieNow(msg, kw)
+
+    app.die = die
     app.name = name
     app.name_clean = tools.clean_env_key(name)
     app.is_initted = True
@@ -233,7 +242,10 @@ def run_app(
         # certain tests require starting the app when already an app is running:
         if call_main_when_already_running:
             return main()
-        app.die('Repeated call of app.run_app', hint='Should call only once per proc.')
+        app.die(
+            'Repeated call of app.run_app',
+            hint='Should call only once per proc.',
+        )
 
     running[0] = True
 
@@ -299,11 +311,32 @@ def define_action_flags_in_cli():
             continue
         have.add(key)
         args[p] = '--' + f['key']
+        # after this all subclass flags for that action are defined:
         tools.define_flags(f['flg_cls'], sub=key, parent_autoshort=f['autoshort'])
-        action[0] = key
+        allow_short_action_cli_args(args, p, key)
+        app.selected_action = key
+        # action[0] = key
 
 
-action = [0]
+def allow_short_action_cli_args(args, p, key):
+    """
+    make 'myaction --filename[=v]' ident to 'myaction --myaction_filename[=v]' cli
+    """
+    L = len(args) - 1
+    while p < L:
+        p += 1
+        a = args[p]
+        if not a[:2] == '--':
+            continue
+        a = a[2:].split('=', 1)
+        s = f'{key}_{a[0]}'
+        if s in FLG.__flags:
+            args[p] = f'--{s}'
+            if len(a) == 2:
+                args[p] += f'={a[1]}'
+
+
+# action = [0]
 # def on_flag_parse_err_have_action_flag(key, af, parser):
 #     p = sys.argv.index(key)
 #     key = af['key']  # short to long
@@ -333,11 +366,13 @@ action = [0]
 
 class DieNow(Exception):
     # required since sys.exit will be catched - halting the app, not stopping it
-    ''
+    def __init__(self, msg, kw):
+        self.msg = msg
+        self.kw = kw
 
 
 class dev_app_exc_handler(abslapp.ExceptionHandler):
-    wants = lambda self, exc: True
+    def wants(self, exc): return True
 
     def handle(self, exc):
         if type(exc) == DieNow:
@@ -371,9 +406,6 @@ def reload_handler(signum, frame):
     raise Reloaded('signal')
 
 
-from io import StringIO
-
-
 # def wrap_flag_parser_with_action_detector(flags_parser):
 #     def parser(args, p=flags_parser):
 #         try:
@@ -446,13 +478,13 @@ def run_phase_2(args, name, main, kw_log, flags_validator, wrapper):
             print('Flags validation error: %s' % err, file=sys.stderr)
             sys.exit(1)
     sl.setup_logging(**kw_log)
-    log = sl.get_logger(name)
+    log = sl.get_logger(app.selected_action or name)
     set_app(name, log)
     watcher_pid = None
     if FLG.dirwatch:
         # TODO: simply do it with entr:
         # cat conf/reload _py3.8
-        ##!/usr/bin/env bash
+        # !/usr/bin/env bash
 
         # ps wwwax |grep python |grep app | grep client | xargs kill
 
@@ -465,29 +497,11 @@ def run_phase_2(args, name, main, kw_log, flags_validator, wrapper):
         cmd = [w, ':'.join([d, str(os.getpid()), match, rec])]
         watcher_pid = subprocess.Popen(cmd).pid
 
-    # An Action class - use it:
-    post = None
-    if isinstance(main, type):
-        action_cls = main
-        if not action[0]:
-            for af in tools.action_flags.values():
-                if af['flg_cls'].d:
-                    action[0] = af['key']
-                    app.debug('Running default action', action=af['key'])
-                    break
-        main = getattr(action_cls, action[0], None)
-        if not main:
-            app.die('Require action')
-        pre = getattr(action_cls, '_pre', 0)
-        if pre:
-            # app.debug('Prepare hook')
-            pre()
-            # app.debug(action[0])
-        post = getattr(action_cls, '_post', 0)
-
-    res = None
+    res = post = None
     while True:
         try:
+            if isinstance(main, type):
+                main, post = handle_action_cls(app, action_cls=main)
             # so that everybody knows what is running. informational
             app._app_func = main
             # main = lambda: run_app(Action, flags=Flags, wrapper=cleanup)
@@ -497,6 +511,9 @@ def run_phase_2(args, name, main, kw_log, flags_validator, wrapper):
                 signal.signal(1, reload_handler)
                 while 1:
                     time.sleep(10)
+        except DieNow as ex:
+            app.error(ex.msg, exc=ex, **ex.kw)
+            raise
         except Reloaded as ex:
             continue
         except SystemExit as ex:
@@ -523,10 +540,15 @@ def run_phase_2(args, name, main, kw_log, flags_validator, wrapper):
     if not isinstance(res, (list, dict, tuple)):
         if not res is None:
             print(res)
-        return res
+        return
     # exit phase. postprocessing, pretty to stdout, plain to | jq .:
     if FLG.flat:
         res = tools.flatten(res, sep='.')
+    f = getattr(app, 'out_formatter', 0)
+    if f:
+        r = f(res)
+        if r:
+            return
     if not sys.stdout.isatty():
         jres = json.dumps(res, default=str)
         print(jres)
@@ -537,18 +559,48 @@ def run_phase_2(args, name, main, kw_log, flags_validator, wrapper):
     # return res
 
 
-class Die(Exception):
-    log = None
+def handle_action_cls(app, action_cls, post=None):
+    # An Action class - use it:
+    if not app.selected_action:
+        for af in tools.action_flags.values():
+            if af['flg_cls'].d:
+                app.selected_action = af['key']
+                app.debug('Running default action', action=af['key'])
+                break
+    if not app.selected_action:
+        app.die('Require action. No default action is set.')
+    main = getattr(action_cls, app.selected_action, None)
+    if not main:
+        app.die('Require action', no_found=app.selected_action)
+    pre = getattr(action_cls, '_pre', 0)
+    if pre:
+        # app.debug('Prepare hook') - may parametrize the action func
+        _ = pre()
 
-    def __init__(self, msg, log=None, **kw):
-        log = log or self.log
-        if log:
-            log.error(msg, **kw)
-        else:
-            print(
-                msg, json.dumps(kw, sort_keys=True, default=str, indent=4)[1:-1],
-            )
-        raise DieNow()
+        if _:
+            main = _
+    if isinstance(main, type):
+        main = main.run
+    post = getattr(action_cls, '_post', 0)
+    return main, post
+
+
+#
+# class Die(Exception):
+#     log = None
+#
+#     def __xinit__(self, msg, log=None, **kw):
+#         raise DieNow(msg, kw)
+
+# log = log or self.log
+# if log:
+#     log.error(msg, **kw)
+# else:
+#     print(
+#         msg, json.dumps(kw, sort_keys=True, default=str, indent=4)[1:-1],
+#     )
+# breakpoint()  # FIXME BREAKPOINT
+# raise DieNow()
 
 
 def app_func(inner=False):
@@ -594,6 +646,8 @@ def do(
 
 
 def system(cmd, no_fail=False):
+    if isinstance(cmd, list):
+        cmd = ' '.join('"%s"' % str(i) for i in cmd)
     # print('\x1b[38;5;240m', end='')
     d = ' 1>&2'
     # cat -> colors off
@@ -601,6 +655,7 @@ def system(cmd, no_fail=False):
     rcmd = 'echo -ne "\x1b[38;5;240m"%s; %s%s || touch "%s"; echo -ne "\x1b[0m"%s'
     rcmd = rcmd % (d, cmd, d, fnf, d)
     os.system(rcmd)
+
     err = False
     if os.path.exists(fnf):
         os.unlink(fnf)
@@ -617,4 +672,8 @@ def system(cmd, no_fail=False):
 # is set into app as .die:
 # allows raise app.die(msg, **kw) with correct error logging:
 # we want to raise for --pdb_post_mortem
-app_die = lambda app: type('Die', (Die,), {'log': app.log})
+# app_die = lambda app: type('Die', (Die,), {'log': app.log})
+
+i = 1
+
+i = 1
